#include <p18cxxx.h>
#include <timers.h>

// CPU System Clock Postscaler 1:1
// CPU Clock = ( ( OSC Input * 4 )
// Disabld WDT , Enable Stack Overflow/Underflow Reset , disable Extended Instruction   
#pragma config FOSC = HS , PLLEN = ON , CPUDIV = NOCLKDIV
#pragma config WDTEN = OFF , STVREN = ON , XINST = OFF , LVP = OFF

#define SystemFrequency		48000000L / 4
#define Timer2Tick			SystemFrequency / 16 / 1 / Timer2TogglesPerSec  
#define Timer2TogglesPerSec	10000

void ISR_High( void );
void ISR_Low( void );

void main( void )
{
	// External Crtstal
	OSCCONbits.SCS0 = 0;		// Primary clock is system clock
	OSCCONbits.SCS1 = 0;		// Primary clock is system clock

	// Enable PLL
	OSCTUNEbits.SPLLEN = 1;

	TRISCbits.TRISC4 = 0;
	TRISCbits.TRISC5 = 0;

	LATCbits.LATC4 = 1;
	LATCbits.LATC5 = 0;

	TRISCbits.TRISC0 = 0;
	TRISCbits.TRISC1 = 0;
	TRISCbits.TRISC2 = 0;

	LATCbits.LATC0 = 0;
	LATCbits.LATC1 = 0;
	LATCbits.LATC2 = 0;

	ANSELbits.ANS4 = 0;	
	ANSELbits.ANS5 = 0;	
	ANSELbits.ANS6 = 0;	

	OpenTimer2( TIMER_INT_ON & T2_PS_1_16 & T2_POST_1_1 );
	PR2 = Timer2Tick;

	INTCONbits.PEIE = 1;
	INTCONbits.GIE = 1;

    while( 1 );
}

/* High Priority Interrupt Vector */
#pragma code High_Vector = 0x0008
void Interrupt_High_Vector( void )
{
	_asm
	goto ISR_High
	_endasm
}
#pragma code

/* Low Priority Interrupt Vector */
#pragma code Low_Vector = 0x0018
void Interrupt_Low_Vector( void )
{
	_asm
	goto ISR_Low
	_endasm
}
#pragma code

/* High Priority ISR */
#pragma interrupt ISR_High
void ISR_High( void )
{
	if( PIR1bits.TMR2IF && PIE1bits.TMR2IE )
	{
		static unsigned int i = 0 , j = 0 , k = 0 , l = 0;

		PIR1bits.TMR2IF = 0;

		if( ++i > 4999 )
		{
			i = 0;

			// LEDAC
			LATCbits.LATC4 = !LATCbits.LATC4;
			LATCbits.LATC5 = !LATCbits.LATC5;
		}

		if( ++j > 9 )
		{
			j = 0;
			LATCbits.LATC0 = !LATCbits.LATC0;
		}

		if( ++k > 49 )
		{
			k = 0;
			LATCbits.LATC1 = !LATCbits.LATC1;
		}

		if( ++l > 99 )
		{
			l = 0;
			LATCbits.LATC2 = !LATCbits.LATC2;
		}

	}
}
	
/* Low Priority ISR */
#pragma interruptlow ISR_Low
void ISR_Low( void )
{
	while( 1 );
}
